@@ -2,9 +2,12 @@
 Defines default L{AccAdapt.Adapter}s for the L{LSRInterfaces.IEventHandler} 
 interface on L{pyLinAcc.Accessible} objects.
 
-@var event_to_method_map: Maps event names to method names to be called to 
+@var EVENT_HANDLERS: Maps event names to method names to be called to 
   handle them
-@type event_to_method_map: dictionary
+@type EVENT_HANDLERS: dictionary
+@var AE_MAP: Mapping from L{AEEvent}s to raw events that must be registered
+  to generate them
+@type AE_MAP: dictionary
 
 @author: Pete Brunet
 @author: Peter Parente
@@ -25,9 +28,41 @@ from LSRInterfaces import *
 from pyLinAcc import Interfaces, Constants
 import pyLinAcc
 
-event_to_method_map = {
+# opt: AE to raw event mapping, only registers for raw events when desired
+AE_MAP = {ViewChange : ['window:activate', 'window:deactivate', 
+                        'window:create', 'window:destroy'],
+          FocusChange : ['focus', 'object:state-changed:focused'],
+          CaretChange : ['object:text-caret-moved',
+                         'object:text-changed'],
+          ChildrenChange : ['object:children-changed'],
+          PropertyChange : ['object:property-change'],
+          SelectorChange : ['object:text-selection-changed',
+                            #'object:selection-changed',
+                            'object:active-descendant-changed'],
+          # opt: only register for state events that might be worth reporting 
+          # to a script for the time being, lots of state change processing
+          # can really slow down the screen reader
+          # @todo: PP: fill these out and move to an adapter method
+          StateChange : ['object:state-changed:expanded', 
+                         'object:state-changed:checked',
+                         'object:state-changed:enabled',
+                         'object:state-changed:animated',
+                         'object:state-changed:busy'],
+          TableChange : ['object:row-inserted',
+                         'object:row-deleted',
+                         'object:row-reordered',
+                         'object:column-inserted',
+                         'object:column-deleted',
+                         'object:column-reordered'],
+          ScreenChange : ['object:bounds-changed',
+                          'object:text-bounds-changed',
+                          'object:visible-data-changed']
+          }
+
+EVENT_HANDLERS = {
   'window:activate': '_handleViewChange',
   'focus' : '_handleFocusEvent',
+  'object:state-changed:focused' : '_handleFocusEvent',
   'object:text-changed:insert' : '_handleTextEvent',
   'object:text-changed:delete' : '_handleTextEvent',
   'object:text-caret-moved' : '_handleCaretEvent',
@@ -74,11 +109,85 @@ class DefaultEventHandlerAdapter(Adapter):
   @type last_focus: pyLinAcc.Accessible
   '''
   provides = [IEventHandler]
-  # event are handled serially, no need to make lots of adapters
-  #singleton = True
-  
+  singleton = True
   last_focus = None
   
+  def getRawEvents(self, kind):
+    '''
+    Gets a list of raw AT-SPI event names that map to the given kind of 
+    L{AEEvent}.
+    
+    @param kind: Indicates the type of L{AEEvent} some part of the system wants
+      to be able to process
+    @type kind: L{AEEvent} class
+    @return: List of AT-SPI event names
+    @rtype: list of string
+    @raise KeyError: When no mapping exists for the given event
+    '''
+    return AE_MAP[kind]
+  
+  @pyLinAcc.errorToLookupError
+  def getAEViewEvents(self, event, collector, vm):
+    '''
+    Determines if the active view has changed and what events need to be
+    fired in response. The possible cases include a normal view change 
+    (app1 window to app2 window), a floating widget change (app1 window to
+    app1 floater), or a overlay change (app1 window to app2 floater). Resets 
+    the last focus when the third case occurs so that the first application
+    can announce its restored focus when the floater goes away. For instance,
+    if the metacity task switcher appears and the user immediately closes it,
+    he lands right back in the window where he started. In this case, the
+    new focus is the last focus, but we still want to announce it.
+    
+    @param event: Raw event
+    @type event: L{pyLinAcc.Event.Event}
+    @param collector: Callable object that collects L{AEEvent}s to process. 
+      Accepts N parameters of type L{AEEvent}.
+    @type collector: callable
+    @param vm: Reference to the view manager that has information about the 
+      current view and needs to store a reference to a new view
+    @rtype vm: L{ViewManager}
+    '''
+    try:
+      # make sure we can get the event type and role
+      et = event.type.major
+      role = event.source.getRole()
+    except Exception:
+      # bad event
+      return
+    
+    # get if this is a window
+    is_win = role == Constants.ROLE_WINDOW
+    
+    if et == 'activate':
+      if vm.setView(event.source):
+        # only a view change if the event source wasn't already the active view
+        vm.setActivated(True)
+        collector(ViewChange(vm.getView(), ViewChange.GAINED))
+    elif vm.getView() is not None and et == 'deactivate':
+      por = POR(event.source)
+      if por == vm.getView():
+        # only unset the flag if another activate hasn't been received
+        vm.setActivated(False)
+      collector(ViewChange(por, ViewChange.LOST))
+    elif vm.getActivated() and is_win:
+      # the active application created a floating window
+      if et == 'create':
+        vm.setView(event.source)
+      elif et == 'destroy':
+        # @todo: PP: need to refocus last focused control or fix gail bug
+        pass
+    elif not vm.getActivated() and is_win:
+      # some other application created a floating window
+      if et == 'create':
+        if vm.setView(event.source):
+          collector(ViewChange(vm.getView(), ViewChange.GAINED))
+          # forget about the last focus so that a focus event can be fired when
+          # the previous application is activated
+          DefaultEventHandlerAdapter.last_focus = None
+      elif et == 'destroy':
+        collector(ViewChange(POR(event.source), ViewChange.LOST))
+
   @pyLinAcc.errorToLookupError
   def getAEEvents(self, event, collector):
     '''
@@ -98,51 +207,74 @@ class DefaultEventHandlerAdapter(Adapter):
     try:
       acc = self.subject
       ss = acc.getState()
-      role = acc.getRoleName()
-      if ss.contains(Constants.STATE_DEFUNCT) or role == 'redundant object':
-        return None
+      role = acc.getRole()
+      if (ss.contains(Constants.STATE_DEFUNCT) or 
+          role == Constants.ROLE_REDUNDANT_OBJECT):
+        return
     except AttributeError:
-      return None
+      return
     
     try:
       # try to hash against an exact method
-      name = event_to_method_map[event.type.name]
+      name = EVENT_HANDLERS[event.type.name]
       method = getattr(self, name)
     except (KeyError, AttributeError):
       # if that fails, try to check the major event type
       if event.type.major == 'state-changed':
         method = self._handleStateEvent
       else:
-        return None
+        return
       
-    # synthesize focus lost events
-    if event.type.klass == 'focus':
-      lf = DefaultEventHandlerAdapter.last_focus
-      if lf is not None:
-        events = self._handleUnfocusEvent(lf)
-        if events: collector(*events)
-      DefaultEventHandlerAdapter.last_focus = event
+    if method == self._handleFocusEvent:
+      if self._filterFocusEvent(event):
+        # filter unwanted focus events
+        return
 
     # call the method that handles this event
     # keyword arguments will be passed as-is to AEEvent constructors
     events = method(event, focused=ss.contains(Constants.STATE_FOCUSED) or
                     ss.contains(Constants.STATE_ACTIVE))
-    if events: collector(*events)
-      
-  def _handleUnfocusEvent(self, event, **kwargs):
+
+    if events: 
+      if method == self._handleFocusEvent:
+        # synthesize focus lost events
+        lf = DefaultEventHandlerAdapter.last_focus
+        if lf is not None:
+          lost_events = self._handleUnfocusEvent(lf)
+          if lost_events: collector(*lost_events)
+        DefaultEventHandlerAdapter.last_focus = event.source
+      # send the generated events
+      collector(*events)
+
+  def _handleUnfocusEvent(self, source, **kwargs):
     '''
     Creates an L{AEEvent.FocusChange} indicating that the accessible being
     adapted has lost the focus.
     
-    @param event: Raw focus change event
-    @type event: L{pyLinAcc.Event.Event}
+    @param event: Source of the last raw focus change event
+    @type event: L{pyLinAcc.Accessible}
     @param kwargs: Parameters to be passed to any created L{AEEvent}
     @type kwargs: dictionary
     @return: L{AEEvent.FocusChange}
     @rtype: tuple of L{AEEvent}
     '''
-    por = POR(event.source, None, 0)
+    por = POR(source, None, 0)
     return (FocusChange(por, False, **kwargs),)
+  
+  def _filterFocusEvent(self, event):
+    '''
+    Determines if a focus event is a repeat on an already focused object or
+    not.
+    
+    @param event: Source of the last raw focus change event
+    @type event: L{pyLinAcc.Accessible}
+    @return: True if this method believes the event should be ignored, False if
+      not
+    @rtype: boolean
+    '''
+    return ((event.type.klass != 'focus' and not event.detail1) or 
+            DefaultEventHandlerAdapter.last_focus == event.source or
+            event.source.getRole() == Constants.ROLE_PAGE_TAB_LIST)
 
   def _handleFocusEvent(self, event, **kwargs):
     '''
