@@ -31,8 +31,8 @@ class DefaultAccInfoAdapter(PORAdapter):
   @cvar VISIBLE_STATES: States that an accessible must have to be considered 
     visible
   @type VISIBLE_STATES: tuple
-  @cvar TRIVIAL_STATES: States that an accessible must not have to be considered
-    non-trivial
+  @cvar TRIVIAL_STATES: States that an accessible must not have to be
+    considered non-trivial
   @type TRIVIAL_STATES: tuple
   @cvar TRIVIAL_ROLES: Roles that an accessible must not have to be considered
     non-trivial
@@ -237,6 +237,35 @@ class DefaultAccInfoAdapter(PORAdapter):
     '''
     raise NotImplementedError
 
+  @pyLinAcc.errorToLookupError
+  def getAccVisualExtents(self):
+    '''
+    Gets the size of the bounding box of an accessible.
+
+    @return: Width and height
+    @rtype: 2-tuple of integer
+    @raise NotImplementedError: When this accessible does not support the
+      Component interface
+    '''
+    c = Interfaces.IComponent(self.accessible)
+    bb = c.getExtents(Constants.DESKTOP_COORDS)
+    return bb.width, bb.height
+
+  @pyLinAcc.errorToLookupError
+  def getAccVisualPoint(self):
+    '''
+    Gets the visual position of the focal point wihin an accessible object,
+    the center in a simple box-like control.
+
+    @return: x,y coordinate
+    @rtype: 2-tuple integer
+    @raise NotImplementedError: When this accessible does not support the
+      Component interface
+    '''
+    c = Interfaces.IComponent(self.accessible)
+    bb = c.getExtents(Constants.DESKTOP_COORDS)
+    return (bb.x+bb.width)/2, (bb.y+bb.height)/2
+
   @pyLinAcc.errorToLookupError
   def getAccTextAttr(self, name):
     '''
@@ -404,7 +433,11 @@ class DefaultAccInfoAdapter(PORAdapter):
     app = self.accessible.getApplication()
     if app is None:
       raise LookupError
-    return unicode(app.name, 'utf-8')
+    try:
+      return unicode(app.name, 'utf-8')
+    except UnicodeDecodeError:
+      #363431: protect against junk app names
+      return ''
 
   @pyLinAcc.errorToLookupError
   def getAccAppID(self):
@@ -745,4 +778,4 @@ class DefaultAccInfoAdapter(PORAdapter):
     @raise LookupError: When the accessible object is dead
     '''
     parent = IAccessibleNav(self.subject).getParentAcc()
-    return IAccessibleInfo(parent).allowsAccEmbeds()
\ No newline at end of file
+    return IAccessibleInfo(parent).allowsAccEmbeds()
