@@ -119,6 +119,43 @@ class TextAccInfoAdapter(DefaultAccInfoAdapter):
     #else:
       #return _('text line')
 
+  @pyLinAcc.errorToLookupError
+  def getAccVisualExtents(self):
+    '''
+    Gets the visual width and height of a line or the entire text area if
+    item offset is None.
+
+    @return: Width and height extents
+    @rtype: 2-tuple of integer
+    @raise LookupError: When the accessible object is dead
+    '''
+    it = Interfaces.IText(self.accessible)
+    if self.item_offset is None:
+      return super(TextAccInfoAdapter, self).getVisualExtents()
+    line, s, e = text.getTextAtOffset(self.item_offset, 
+                                      Constants.TEXT_BOUNDARY_LINE_START)
+    xs, ys, ws, hs = it.getCharacterExtents(s, Constants.DESKTOP_COORDS)
+    xe, ye, we, he = it.getCharacterExtents(e, Constants.DESKTOP_COORDS)
+    return (xe-xs), (ye-ys)
+
+  @pyLinAcc.errorToLookupError
+  def getAccVisualPoint(self):
+    '''
+    Gets the center of a line or the entire text area if item offset is None.
+
+    @return: x,y coordinates
+    @rtype: 2-tuple of integer
+    @raise LookupError: When the accessible object is dead
+    '''
+    it = Interfaces.IText(self.accessible)
+    if self.item_offset is None:
+      return super(TextAccInfoAdapter, self).getVisualPoint()
+    # compute the offset in the line
+    off = self.item_offset + self.char_offset
+    line, s, e = text.getTextAtOffset(off, Constants.TEXT_BOUNDARY_LINE_START)
+    x, y, w, h = it.getCharacterExtents(s, Constants.DESKTOP_COORDS)
+    return x, ((y+h)/2)
+
 class TextNavAdapter(DefaultNavAdapter):
   '''
   Overrides L{DefaultNavAdapter} to provide navigation over text lines as 
