@@ -13,13 +13,11 @@ All rights reserved. This program and the accompanying materials are made
 available under the terms of the Common Public License v1.0 which accompanies
 this distribution, and is available at
 U{http://www.opensource.org/licenses/cpl1.0.php}
-
-@todo: DBC: add methods to play sound, ie. play(dev, file)
 '''
 import logging, sys
-import AEOutput, AEInput, AEEvent
+import AEOutput, AEInput, AEEvent, AEMonitor, UIRegistrar
 from LSRInterfaces import implements
-from UIRegistrar import UIRegistrar, DEVICE, MONITOR
+from UIRegistrar import DEVICE, MONITOR
 from i18n import _
 import LSRConstants
 
@@ -33,14 +31,6 @@ class DeviceManager(object):
   from output styles to semantic concepts in the user interface to allow
   customization of how information is presented.
   
-  @ivar dirty_style: Tracks whether a style has been fetched or not for 
-    modification. If it has, the L{sendString} should also send the style to
-    a device.
-  @type dirty_style: boolean
-  @ivar last_style: The last L{AEOutput.Style} object sent to a device. Used
-    to determine if the next L{sendString} should send a style or not. Not
-    sending a style is an optimization.
-  @type last_style: L{AEOutput.Style}
   @ivar event_manager: L{EventManager} to which this L{DeviceManager} can post 
     L{AEEvent}s 
   @type event_manager: L{EventManager}
@@ -48,20 +38,18 @@ class DeviceManager(object):
   @type out_devs: list
   @ivar in_devs: All input devices registered and initialized
   @type in_devs: list
-  @ivar out_mons: All output monitors registered and initialized
-  @type out_mons: list
-  @ivar in_mons: All input monitors registered and initialized
-  @type in_mons: list
+  @ivar in_mons: Collection of monitors to notify about input
+  @type in_mons: L{AEMonitor.MonitorCollection}
+  @ivar out_mons: Collection of monitors to notify about output
+  @type out_mons: L{AEMonitor.MonitorCollection}
   @ivar def_output: The "default" output device
   @type def_output: L{AEOutput}
   @ivar def_input: The "default" input device
   @type def_input: L{AEInput}
   @ivar acc_eng: The AccessEngine reference that created this class
   @type acc_eng: L{AccessEngine}
-  @ivar styles: Dictionary of L{AEOutput.Style}s populated on device 
-    registration in L{_registerOutputDevice}. Keys are a 3-tuple of device,
-    semantic, and layer.
-  @type styles: dictionary of L{AEOutput.Style}
+  @ivar marks: Maps index markers on devices to 
+  @type marks: dictionary
   '''
   def __init__(self, acc_eng):
     '''
@@ -72,141 +60,126 @@ class DeviceManager(object):
     '''
     self.out_devs = []
     self.in_devs = []
-    self.out_mons = []
-    self.in_mons = []
+    self.in_mons = AEMonitor.MonitorCollection()
+    self.out_mons = AEMonitor.MonitorCollection()
     self.def_output = None
     self.def_input = None
     self.acc_eng = acc_eng
     self.event_manager = None
-    self.styles = {}
-    self.dirty_style = True
-    self.last_style = None
+    self.settings_manager = None
+    self.marks = {}
 
-  def init(self, event_man, **kwargs):
+  def init(self, event_man, sett_man, **kwargs):
     '''
     Called by L{AccessEngine} at startup.
-    
-    @todo: PP: how do we unload these devices when they're no longer desired?
 
     @param event_man: L{EventManager} to which this L{DeviceManager} can
       post L{AEEvent}s 
     @type event_man: L{EventManager}
+    @param sett_man: L{SettingsManager} which this L{DeviceManager} can use to
+      load and persist device settings
+    @type sett_man: L{EventManager}
     @param kwargs: References to managers not of interest here
     @type kwargs: dictionary
     '''
     self.event_manager = event_man
-    reg = UIRegistrar()
+    self.settings_manager = sett_man
+    
+    # load all I/O monitor classes to be created at startup
+    mons = UIRegistrar.loadAssociated(MONITOR, self.acc_eng.getProfile())
+    self.addMonitors(*mons)
+        
     # load all device classes to be created at startup
-    devs = reg.loadAssociated(DEVICE, self.acc_eng.getProfile())
+    self.loadDevices()
+    
+  def loadDevices(self):
+    '''
+    Iterates over all devices in the active profile and loads ones that provide
+    functionality different from those already loaded.
+    '''
+    devs = UIRegistrar.loadAssociated(DEVICE, self.acc_eng.getProfile())
     # register all devices, making the first input and output the defaults
     for d in devs:
       # call registerDevice which will automatically set defaults
       try:
         self.registerDevice(d)
       except (AEOutput.AEOutputError, AEInput.AEInputError):
-        log.info('Could not initialize device: %s', str(d))
-      except NotImplementedError:
-        log.info('Device does not provide an I/O interface: %s', str(d))
-    # load all I/O monitor classes to be created at startup
-    mons = reg.loadAssociated(MONITOR, self.acc_eng.getProfile())
-    for m in mons:
-      # call registerDevice which will weed out non-IO monitors for us
-      try:
-        self.registerMonitor(m)
-      except NotImplementedError:
-        # ignore NotImplementedErrors if we get other monitor types in the mix
-        pass
-  
-    # make sure we initialized at least one input and one output device
-    if self.def_input is None:
-      self.close()
-      raise ValueError(_('No input device available'))
+        log.debug('could not initialize device: %s', d.getName())
+      except NotImplementedError, e:
+        log.debug('device %s does not provide a new interface: %s' % 
+                 (d.getName(), str(e)))
+    
+    # warn if we didn't initialize at least one output device
     if self.def_output is None:
-      self.close()
-      raise ValueError(_('No output device available'))
+      log.warn(_('no output available'))
+    if self.def_input is None:
+      log.warn(_('no input available'))
+      
+  def unloadDevices(self):
+    '''
+    Unloads all current devices.
+    '''
+    for d in self.out_devs:
+      self.unregisterDevice(d)
+    # @todo: PP: support unloading of input devices
+    #for d in self.in_devs:
+      #self.unregisterDevice(d)
+    self.def_output = None
+    #self.def_input = None
     
   def close(self):
     '''
     Shuts down this manager and all its registered L{AEOutput} and L{AEInput}
-    devices.
+    devices and L{AEMonitor}s. Saves all style information for currently 
+    loaded devices.
     '''
     for d in self.out_devs:
-      try:
-        d.close()
-      except:
-        # ignore all exceptions
-        pass
+      self.unregisterDevice(d)
     for d in self.in_devs:
-      try:
-        d.close()
-      except:
-        # ignore all exceptions
-        pass
-      
+      self.unregisterDevice(d)
+    self.def_output = None
+    self.def_input = None
+    self.in_mons.clear()
+    self.out_mons.clear()
+
   def sendStop(self, dev, sem, layer):
     '''
     Sends the stop command to the referenced output device. When the device has
     not been registered or is None, the stop command is sent to the default
-    output device. After sending the stop, L{showStop} is called.
+    output device.
 
     @param dev: The device to send the stop command or None to send to L{the 
       default output device<setDefaultOutput>}.
     @type dev: L{AEOutput}
-    @param sem: Semantic description of the information to stop None to indicate
-      stopping all output
+    @param sem: Semantic description of the information to stop None to 
+      indicate stopping all output
     @type sem: integer
-    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
-      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @param layer: Layer on which the event occurred, one of 
+      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
     @type layer: integer
     '''
-    # when dev is None assume defaultOutput (which may also be None)
+    # when dev is None assume default output
     dev = dev or self.def_output
+    # notify monitors
+    self.out_mons.show(event=OutputEvent.STOP, dev=dev, sem=sem, layer=layer)
+    if dev is None: return
     if sem is None and layer is None:
-      self.showStop(dev, sem, layer, None)
+      # send a global stop
       dev.sendStop(None)
     elif sem is None:
       # find all styles associated with the given layer
-      all = set([self.styles[(sem, layer, dev)] for sem in
+      all = set([dev.getStyle((sem, layer)) for sem in
                  LSRConstants.SEMANTIC_STYLES.keys()])
-      # send stops to all associated
-      for s in all:
-        self.showStop(dev, sem, layer, s)
-        dev.sendStop(s)
+      map(dev.sendStop, all)
     else:
-      s = self.styles[(sem, layer, dev)]
+      s = dev.getStyle((sem, layer))
       dev.sendStop(s)
-      self.showStop(dev, sem, layer, s)
-    # reset the last style flag so that it is resent on the next sendString,
-    # otherwise, the stop my prempt the application of the last style
-    self.last_style = None
-
-  def showStop(self, dev, sem, layer, style):
-    '''
-    Calls all registered output monitors to show that a stop command has been
-    sent to the referenced output device. This does nothing when no monitors
-    are registered.
-
-    @param dev: The device that was sent the stop command
-    @type dev: L{AEOutput}
-    @param sem: Semantic description of the output that was stopped
-    @type sem: integer
-    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
-      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
-    @type layer: integer
-    @param style: Style associated with the semantic 
-    @type style: L{AEOutput.Style}
-    '''
-    for mon in self.out_mons:
-      mon.showStop(dev, sem, layer, style)
-    # for demonstration purposes, just print if no monitors
-    #if not self.out_mons:
-    #  sys.stdout.write('{stop}')
 
   def sendTalk(self, dev, sem, layer):
     '''
     Tells the specified output device to send buffered data. When the device
     has not been registered or is None, the talk command is sent to the
-    default output device. After sending the talk, L{showTalk} is called.
+    default output device.
 
     @param dev: The device to send the talk command or None to send to L{the 
       default output device<setDefaultOutput>}
@@ -214,48 +187,69 @@ class DeviceManager(object):
     @param sem: Semantic information to start outputting or None to indicate 
       that all buffered information should be output
     @type sem: integer
-    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
-      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @param layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
+      L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
     @type layer: integer
     '''
-    # when dev is None assume defaultOutput (which may also be None)
+    # when dev is None assume default output
     dev = dev or self.def_output
+    # notify monitors
+    self.out_mons.show(event=OutputEvent.TALK, dev=dev, sem=sem, layer=layer)
+    if dev is None: return
     if sem is None:
       s = None
     else:
-      s = self.styles[(sem, layer, dev)]
+      s = dev.getStyle((sem, layer))
     dev.sendTalk(s)
-    # always send the command to the registered monitors
-    self.showTalk(dev, sem, layer, s)
 
-  def showTalk(self, dev, sem, layer, style):
+  def sendFilename(self, dev, filename, sem, layer, talk=True):
     '''
-    Calls all registered output monitors to show the talk command sent to the
-    referenced output device. This does nothing when no monitors are
-    registered.
+    Sends the filename to the specified output device. When the device has not
+    been registered or is None, the string is sent to the default output
+    device.
 
-    @param dev: The device that was sent the talk command.
+    @param dev: The device to send the string or None to send to L{the default 
+      output device<setDefaultOutput>}
     @type dev: L{AEOutput}
-    @param sem: Semantic information that was output
+    @param filename: The filename to send to the referenced device
+    @type filename: string
+    @param sem: Semantic description of the text to send or None to indicate the
+      information is void of any known semantic
     @type sem: integer 
-    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
-      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @param layer: Layer on which the event occurred, one of L{Task.LAYER_FOCUS},
+      L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
     @type layer: integer
-    @param style: Style associated with the semantic 
-    @type style: L{AEOutput.Style}
+    @param talk: Defaults to True to indicate that L{sendTalk} should be called
+      after all characters are sent
+    @type talk: boolean
     '''
-    for mon in self.out_mons:
-      mon.showTalk(dev, sem, layer, style)
-    # for demonstration purposes, just print if no monitors
-    #if not self.out_mons:
-    #  print '{talk}'
+    # when dev is None assume default output
+    dev = dev or self.def_output
+    # notify monitors
+    self.out_mons.show(event=OutputEvent.FILENAME, text=filename, dev=dev, 
+                       sem=sem, layer=layer)
+    if dev is None: return
+    # find the style
+    style = dev.getStyle((sem, layer))
+    # respect muting
+    if style.Mute: return
+    try:
+      dev.sendFilename(filename, style)
+      if talk:
+        self.sendTalk(dev, sem, layer)
+    except NotImplementedError:
+      pass
+    # make the style as clean
+    style.makeClean()
     
-  def sendString(self, dev, text, sem, layer, talk=True):
+  def sendString(self, dev, text, sem, layer, talk=True, por=None):
     '''
     Sends the string to the specified output device. When the device has not
     been registered or is None, the string is sent to the default output
     device. The string is sent with a L{AEOutput.Base.AEOutput.sendString} call
     followed by L{sendTalk} when the talk parameter is True.
+    
+    @todo: PP: collect word markers
 
     @param dev: The device to send the string or None to send to L{the default 
       output device<setDefaultOutput>}
@@ -265,68 +259,51 @@ class DeviceManager(object):
     @param sem: Semantic description of the text to send or None to indicate the
       information is void of any known semantic
     @type sem: integer 
-    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
-      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @param layer: Layer on which the event occurred, one of 
+      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
     @type layer: integer
     @param talk: Defaults to True to indicate that L{sendTalk} should be called
       after all characters are sent
     @type talk: boolean
+    @param por: Point of regard to the start of the string if one exists or 
+      None if the string did not originate from a L{POR}
+    @type por: L{POR}
     '''
-    # when dev is None assume defaultOutput (which may also be None)
+    # when dev is None assume default output
     dev = dev or self.def_output
-    s = self.styles[(sem, layer, dev)]
+    # notify monitors
+    self.out_mons.show(event=OutputEvent.STRING, text=text, dev=dev,
+                       sem=sem, layer=layer)  
+    if dev is None:
+      # make sure we log the talk, even if no device exists
+      if talk:
+        self.sendTalk(dev, sem, layer)
+      return
+    # get the style
+    style = dev.getStyle((sem, layer))
     # respect muting
-    if s.Mute: return
-    # if the style hasn't changed, and the last style is the same one as the
-    # one we're trying to send now, don't resend it
-    if not self.dirty_style and self.last_style == s:
-      s = None
-    else:
-      # keep track of the last style sent and reset the dirty flag
-      self.last_style = s
-      self.dirty_style = False
-    dev.sendString(text, s)
+    if style.Mute: return
+    # parse the string using the current style
+    parsed = dev.parseString(text, style, por)
+    # send all words, pors, and styles
+    for word, por, new_style in parsed:
+      # send the index and collect the marker
+      #try:
+        #marker = dev.sendIndex(style)
+      #except NotImplementedError:
+        #pass
+      # send the word string
+      dev.sendString(word, new_style)
     if talk:
-       self.sendTalk(dev, sem, layer)
-    self.showString(dev, text, sem, layer, s, talk)
+      self.sendTalk(dev, sem, layer)
+    # mark the style as clean
+    style.makeClean()
 
-  def showString(self, dev, text, sem, layer, style, talk):
-    '''
-    Sends a string to all registered output monitors. This method should be
-    used when only monitor output is desired without sending to any devices.
-
-    This is also called by L{sendString} when no output devices have been
-    registered to support testers with no other output.
-
-    @param dev: The device that was sent the string.
-    @type dev: L{AEOutput}
-    @param text: The text that was sent to the referenced device
-    @type text: string
-    @param sem: Semantic description of the text sent
-    @type sem: integer
-    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
-      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
-    @type layer: integer
-    @param style: Style associated with the semantic 
-    @type style: L{AEOutput.Style}
-    @param talk: Indicates that L{sendTalk} was called
-    @type talk: boolean
-    '''
-    for mon in self.out_mons:
-      mon.showString(dev, text, sem, layer, style, talk)
-    # for demonstration purposes, just print if no monitors
-    #if not self.out_mons:
-      #if talk:
-        #print text
-      #else:
-        #for ch in text:
-          #sys.stdout.write(ch)
-
-  def sendIndex(self, dev, sem, layer, por):
+  def sendIndex(self, dev, sem, layer, data=None):
     '''
     Sends the referenced index marker to the referenced device. When the
     device has not been registered or is None, the index marker is sent to the
-    default output device. After sending the command, L{showIndex} is called.
+    default output device.
 
     @param dev: The device to send the index marker or None to send to L{the 
       default output device<setDefaultOutput>}
@@ -334,51 +311,53 @@ class DeviceManager(object):
     @param sem: Semantic description of the index text to send or None to 
       indicate the information is void of any known semantic
     @type sem: integer
-    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
-      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @param layer: Layer on which the event occurred, one of 
+      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
     @type layer: integer
-    @param por: Point of regard associated with the index marker
-    @type por: L{POR}
-
-    @todo: PP: rewrite for POR
+    @param data: Data to be forwarded to any observers when the index is 
+      reached
+    @type data: object
     '''
-    # when dev is None assume defaultOutput (which may also be None)
-    if dev is None:
-      dev = self.def_output
+    # when dev is None assume default output
+    dev = dev or self.def_output
+    # notify registered monitors
+    self.out_mons.show(event=OutputEvent.INDEX, dev=dev, sem=sem,layer=layer)
+    if dev is None: return
+    # get the style
+    s = dev.getStyle((sem, layer))
+    # respect muting
+    if s.Mute: return
     try:
-      if dev in self.out_devs:
-        dev.sendIndex(marker)
-      # always send the command to the registered monitors
-      self.showIndex(dev, sem, layer, marker)
+      # generate a new marker on the device
+      mark = dev.sendIndex(s)
     except NotImplementedError:
       # do nothing when indexing not implemented
+      return
+    # store the mark and associated data for later retrieval
+    #self.marks[(dev, mark)] = (sem, layer, data)
+
+  def sendWelcome(self):
+    '''Outputs the LSR welcome message on the default output device.'''
+    msg = _('welcome to %s version %s, revision %s') % \
+        (LSRConstants.NAME, LSRConstants.PROG_VERSION, LSRConstants.PROG_DATE)
+    try:
+      st = self.def_output.getStyle((LSRConstants.SEM_ITEM, 
+                                     AEEvent.LAYER_FOCUS))
+    except AttributeError:
+      # no output available
+      return
+    self.def_output.sendString(msg, st)
+    self.def_output.sendTalk()
+    
+  def sendGoodbye(self):
+    '''Outputs the LSR goodbye message on the default output device.'''
+    try:
+      st = self.def_output.getStyle((LSRConstants.SEM_ITEM, 
+                                     AEEvent.LAYER_FOCUS))
+      self.def_output.sendStringSync(_('lsr exiting'), st)
+    except AttributeError:
       pass
 
-  def showIndex(self, dev, sem, layer, por):
-    '''
-    Calls all registered output monitors to show the index marker sent to the
-    referenced output device. This does nothing when no output monitors are
-    registered.
-
-    @param dev: The device that was sent the index marker
-    @type dev: L{AEOutput}
-    @param sem: Semantic description of the index text to send or None to 
-      indicate the information is void of any known semantic
-    @type sem: integer
-    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
-      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
-    @type layer: integer
-    @param por: por at which to place the marker
-    @type por: L{POR}
-
-    @todo: PP: rewrite to handle POR
-    '''
-    for mon in self.out_mons:
-      mon.showIndex(dev, sem, layer, por)
-    # for demonstration purposes, just print if no monitors
-    #if not self.out_mons:
-      #sys.stdout.write('{I:%s:%d}' % (dev, marker))
-      
   def _isDuplicateDevice(self, dev, output):
     '''
     Checks if the given device is a duplicate in that it provides no new 
@@ -432,13 +411,13 @@ class DeviceManager(object):
     dev.addInputListener(self._onGesture)
     # append to the in_devs list after successfully initialized
     self.in_devs.append(dev)
-    log.info('Added input device %s', str(dev))
+    log.debug('added input device %s', str(dev))
     # first input device registered is default
     if self.def_input is None:
       self.def_input = dev
     return True
   
-  def _registerOutputDevice(self, dev, num_styles):
+  def _registerOutputDevice(self, dev):
     '''
     Registers the given device as an output device if the device implements the
     L{AEOutput} base class and provides some subinterface not already provided
@@ -448,9 +427,6 @@ class DeviceManager(object):
     
     @param dev: Device to attempt to register
     @type dev: L{AEOutput.AEOutput} class
-    @param num_styles: Number of distinct L{AEOutput.Style}s to requested from
-      the device
-    @type num_styles: integer 
     @return: True if the device was registered, False if it is not an 
       L{AEOutput} device, and None if it provides no new interfaces
     @rtype: boolean or None
@@ -458,7 +434,7 @@ class DeviceManager(object):
     if not implements(dev, AEOutput.AEOutput):
       # don't register something that isn't an output device
       return False
-    dev = dev.getProxy()  
+    dev = dev.getProxy()
     if self._isDuplicateDevice(dev, True):
       # don't register a device that provides all the same interfaces as a
       # previous device
@@ -474,29 +450,43 @@ class DeviceManager(object):
       pass
     # append to the out_devs list after successfully initialized
     self.out_devs.append(dev)
-    log.info('Added output device %s', str(dev))
+    log.debug('added output device %s', str(dev))
     # first output device registered is default
     if self.def_output is None:
       self.def_output = dev
-   
-    # create styles based on the abilities of this device
-    styles = dev.createDistinctStyles(num_styles)
-    # create semantics, layer to styles mapping for device
-    # todo: DBC: later, check to see if previously user-configured
-    for layer, i in enumerate(AEEvent.ALL_LAYERS):
-      # copy the style so we can change the channel
-      s = styles[0].copy()
-      s.Channel = i
-      # provide a default style for a semantic of None
-      self.styles[(None, layer, dev)] = s
-      for sem, grp in LSRConstants.SEMANTIC_STYLES.items():
-        s = styles[grp % num_styles].copy()
-        s.Channel = i
-        # map the style for (semantic, layer, device) using default mapping
-        self.styles[(sem, layer, dev)] = s
-    return True
 
-  def registerDevice(self, dev, num_styles=LSRConstants.DEFAULT_NUM_STYLES):
+    try:
+      # try to load styles for the device
+      dev.postInitOutput(self.settings_manager)
+    except KeyError:
+      # or initialize new output styles
+      self._initOutputStyles(dev)
+    return True
+  
+  def _initOutputStyles(self, dev):
+    '''
+    Initializes styles for an L{AEOutput} device. Calls createDistinctStyles
+    on the device to get an initial batch of styles to use to distinguish
+    some types of information.
+    
+    @param dev: Device reference
+    @type dev: L{AEOutput}
+    '''
+    # create styles based on the abilities of this device
+    num_layers = len(AEEvent.LAYERS_ALL)
+    num_groups = len(LSRConstants.STYLE_GROUP_ALL)
+    styles = dev.createDistinctStyles(num_groups, num_layers)
+    # create semantics, layer to styles mapping for device
+    for layer, i in enumerate(AEEvent.LAYERS_ALL):
+      # provide a default style for a semantic of None
+      s = styles[i*num_groups]
+      dev.setStyle((None, layer), s)
+      for sem, grp in LSRConstants.SEMANTIC_STYLES.items():
+        # provide a style for (semantic, layer, device)
+        s = styles[i*num_groups + (grp % num_groups)]
+        dev.setStyle((sem, layer), s)
+
+  def registerDevice(self, dev):
     '''    
     Registers the referenced device based on its one or more interfaces.
     When the interface is determined, the init() method is called on the device.
@@ -505,58 +495,70 @@ class DeviceManager(object):
     
     @param dev: The device to register
     @type dev: L{AEOutput} or L{AEInput} class
-    @param num_styles: Number of distinct L{AEOutput.Style}s to requested from
-      the device
-    @type num_styles: integer 
     @raise NotImplementedError: When the device implements none of the required
       interfaces
     @raise AEOutputError: When output device initialization fails
     @raise AEInputError: When input device initialization fails
     '''
-    is_out = self._registerOutputDevice(dev, num_styles)
+    is_out = self._registerOutputDevice(dev)
     is_in = self._registerInputDevice(dev)
 
     if is_out == False and is_in == False:
       # raise an exception if none of the required interfaces was implemented
       raise NotImplementedError
-  
-  def registerMonitor(self, mon):
+    
+  def unregisterDevice(self, dev):
     '''
-    @todo: PP: implement when we have input and output monitors
+    Unregisters a device from both the input and output lists based on its
+    capabilities.
+    
+    @param dev: The device to unregister
+    @type dev: L{AEOutput} or L{AEInput} class
     '''
-    pass
-    #implemented = False
-    #registered = False
-    ## maybe register as output monitor
-    #if isinstance(mon, OutputMonitor):
-      #implemented = True
-      #if mon not in self.out_mons:
-        #if not registered:
-          ## no need to multi-init
-          #mon.init()
-        ## append to the in_devs list after successfully initialized
-        #self.out_mons.append(mon)
-        #registered = True
-      #else:
-        #registered = True   # indicate already initialized
+    try:
+      dev.close()
+    except Exception:
+      pass
+    try:
+      # let an output device save its styles
+      dev.postCloseOutput(self.settings_manager)
+    except AttributeError:
+      pass
+    # @todo: PP: allow input devices to save state too
+    try:
+      self.out_devs.remove(dev)
+      log.debug('removed output device %s', dev.getName())
+    except ValueError:
+      pass
+    # @todo: PP: support unloading of input devices
+    try:
+      self.in_devs.remove(dev)
+      log.debug('removed input device %s', dev.getName())
+    except ValueError:
+      pass
+    if self.out_devs:
+      self.def_output = self.out_devs[0]
+    if self.in_devs:
+      self.def_input = self.in_devs[0]
 
-    ## maybe register as input monitor
-    #if isinstance(mon, InputMonitor):
-      #implemented = True
-      #if mon not in self.in_mons:
-        #if not registered:
-          ## no need to multi-init
-          #mon.init()
-        ## append to the in_devs list after successfully initialized
-        #self.in_mons.append(mon)
-        #registered = True
-      #else:
-        #registered = True   # indicate already initialized
-    #if not implemented:
-      ## raise an exception if none of the required interfaces was implemented
-      #raise NotImplementedError
-    ## return whether the device/monitor was registered
-    #return registered
+  def addMonitors(self, *monitors):
+    '''
+    Adds one or more L{AEMonitor}s to the list of monitors to be notified about
+    IO events. 
+    
+    @param monitors: L{AEMonitor}s to notify
+    @type monitors: tuple of L{AEMonitor}s
+    '''
+    self.in_mons.add(InputEvent, monitors)
+    self.out_mons.add(OutputEvent, monitors)
+
+  def getMonitors(self):
+    '''
+    @return: Collections of all loaded input and output L{AEMonitor}s in that
+      order
+    @rtype: 2-tuple L{AEMonitor.MonitorCollection}
+    '''
+    return self.in_mons, self.out_mons
 
   def setDefaultOutput(self, dev):
     '''
@@ -575,7 +577,7 @@ class DeviceManager(object):
       if self.registerDevice(dev):
         self.def_output = dev
     else:
-      raise NotImplementedError(_('Not an output device'))
+      raise NotImplementedError('Not an output device')
 
   def getDefaultOutput(self):
     '''
@@ -597,7 +599,7 @@ class DeviceManager(object):
     @rtype: L{AEOutput}
     '''
     for dev in self.out_devs:
-      if dev.__class__.__name__ == name:
+      if dev.getClassName() == name:
         return dev
     return None
 
@@ -618,7 +620,7 @@ class DeviceManager(object):
       if self.register(dev):
         self.def_input = dev
     else:
-      raise NotImplementedError(_('Not an input device'))
+      raise NotImplementedError('Not an input device')
 
   def getDefaultInput(self):
     '''
@@ -640,28 +642,41 @@ class DeviceManager(object):
     @rtype: L{AEInput}
     '''
     for dev in self.in_devs:
-      if dev.__class__.__name__ == name:
+      if dev.getClassName() == name:
         return dev
     return None
   
-  def _onGesture(self, gesture):
-    '''
-    Creates an L{AEEvent} indicating the given gesture was found on a registered
-    input device. When executed, the L{AEEvent} will notify the L{TierManager}
-    about the gesture and allow it to activate the appropriate L{Task}
-    registered to respond to the gesture in the active L{Tier}.
+  def _onGesture(self, gesture, timestamp):
+    '''    
+    Creates an L{AEEvent} indicating the given gesture was found on a
+    registered input device. When executed, the L{AEEvent} will notify the
+    L{TierManager} about the gesture and allow it to activate the appropriate
+    L{Task} registered to respond to the gesture in the active L{Tier}.
     
     @param gesture: Gesture seen on an input device
     @type gesture: L{AEInput.Gesture}
+    @param timestamp: Time at which the event occurred
+    @type timestamp: float
     '''
-    event = AEEvent.InputGesture(gesture)
+    event = AEEvent.InputGesture(gesture, timestamp)
     self.event_manager.postEvents(event)
+    # notify monitors
+    self.in_mons.show(event=InputEvent.GESTURE, dev=gesture.getDevice(), 
+                      gesture=gesture)
     
-  def _onIndex(self, index):
+  def _onIndex(self, dev, index):
     '''
     @todo: PP: handle index callback markers
     '''
-    pass
+    # notify monitors
+    self.out_mons.show(event=OutputEvent.MARKER)
+    
+  def getDevices(self):
+    '''
+    @return: References to all loaded devices
+    @rtype: list of L{UIElement}
+    '''
+    return list(set(self.in_devs).union(self.out_devs))
   
   def getStyle(self, dev, sem, layer):
     '''
@@ -672,17 +687,83 @@ class DeviceManager(object):
     @param sem: The device semantic from which to get the style, or None to
       get the device's default style.
     @type sem: integer 
-    @param layer: Layer on which the event occurred, one of L{Task.FOCUS_LAYER},
-      L{Task.TIER_LAYER}, or L{Task.BACKGROUND_LAYER}
+    @param layer: Layer on which the event occurred, one of 
+      L{Task.LAYER_FOCUS}, L{Task.LAYER_TIER}, or L{Task.LAYER_BACKGROUND}
     @type layer: integer
     @return: the L{AEOutput.Style} for the given semantic of a device
     @rtype: L{AEOutput.Style}
-    @raise KeyError: When the semantic is invalid
+    @raise KeyError: When the semantic or layer is invalid
     '''
-    self.dirty_style = True
     dev = dev or self.def_output
-    if sem is None:
-      return dev.getDefaultStyle() 
-    else:
-      return self.styles[(sem, layer, dev)]
+    if dev is None: return None
+    if sem is None or layer is None:
+      return dev.getDefaultStyle()
+    return dev.getStyle((sem, layer))
 
+class DeviceEvent(object):
+  '''
+  Base type for indicating IO events to monitors such that a monitor may 
+  select whether to monitor input, output, both, or neither based on this
+  type and its subtypes. An L{AEMonitor} may query a subclass of this class
+  for its event types and their names in order to construct menus allowing the
+  user to select which events should be logged.
+  
+  @cvar default_types: List of device events that should be monitored by 
+    default
+  @type default_types: list of string
+  @cvar all_types: List of all device events
+  @type all_types: list of string
+  '''
+  default_types = []
+  all_types = []
+  
+  @classmethod
+  def getNames(cls):
+    '''
+    @return: Names of all events
+    @rtype: list of string
+    '''
+    return cls.all_types
+  
+  @classmethod
+  def getDefaults(cls):
+    '''
+    @return: Recommended names of events to be logged by default
+    @rtype: list of string
+    '''
+    return cls.default_types
+
+class InputEvent(DeviceEvent):
+  '''
+  Subtype defining constants for input events.
+  
+  @cvar GESTURE: Gesture was seen on an input device
+  @type GESTURE: string
+  '''
+  GESTURE = 'gesture'
+  default_types = []
+  all_types = [GESTURE]
+
+class OutputEvent(DeviceEvent):
+  '''
+  Subtype defining constants for output events.
+  
+  @cvar STRING: String was sent to an output device
+  @type STRING: string
+  @cvar FILENAME: Filename was sent to an output device
+  @type FILENAME: string
+  @cvar TALK: Talk was sent to an output device
+  @type TALK: string
+  @cvar INDEX: Index was sent to an output device
+  @type INDEX: string
+  @cvar MARKER: Marker was encountered during output
+  @type MARKER: string
+  '''
+  STRING = 'string'
+  TALK = 'talk'
+  INDEX = 'index'
+  FILENAME = 'filename'
+  MARKER = 'marker'
+  STOP = 'stop'
+  default_types = [STRING, TALK, FILENAME, STOP]
+  all_types = [STRING, TALK, FILENAME, MARKER, STOP]
