@@ -264,7 +264,7 @@ class Keyboard(AEInput.SystemInput):
         return False
     return True #gesture.getNumActions() != 0
 
-  def _sendKeys(self, gesture, press):
+  def _sendGesture(self, gesture, press):
     '''    
     Synthesizes key strokes for the key codes in the given L{AEInput.Gesture}.
     The presses are noted in L{synth_press} and L{synth_release} so that they
@@ -285,6 +285,23 @@ class Keyboard(AEInput.SystemInput):
         self.synth_press.append(code)
       else:
         self.synth_release.append(code)
+        
+  def _sendKey(self, code, press):
+    '''
+    Synthesizes a single key stroke. The presses are noted in L{synth_press} 
+    and L{synth_release} so that they can be ignored when they are seen in 
+    L{_onKeyPressed} or L{_onKeyReleased} to avoid duplicate processing.
+    
+    @param code: Keycode of the key to generate
+    @type code: integer
+    @param press: Synthesize key press (True) or releas (False)?
+    @type press: boolean
+    '''
+    self.event_manager.generateKeyEvent(code, press)
+    if press:
+      self.synth_press.append(code)
+    else:
+      self.synth_release.append(code)
 
   def _statePossible(self, event, press):
     '''
@@ -320,7 +337,7 @@ class Keyboard(AEInput.SystemInput):
       return self._stateMatch
     else:
       # update the os key state
-      self._sendKeys(self.curr_gesture, True)
+      self._sendGesture(self.curr_gesture, True)
       # move to the invalid state and consume the current key
       event.consume = True
       return self._stateInvalid
@@ -384,7 +401,7 @@ class Keyboard(AEInput.SystemInput):
         return self._stateMatch
       else:
         # send all key presses
-        self._sendKeys(self.curr_gesture, True)
+        self._sendGesture(self.curr_gesture, True)
         # move to the invalid state and eat the key
         event.consume = True
         return self._stateInvalid
@@ -472,7 +489,7 @@ class Keyboard(AEInput.SystemInput):
     if (len(self.curr_gesture) > 1 and self.stateful is not None):
       # build a gesture with the stateful key press only in it
       g = AEInput.Gesture(self, action_codes=[self.stateful])
-      self._sendKeys(g, True)
+      self._sendGesture(g, True)
       self.stateful = None
       
     #print 'state:', self.state
@@ -510,7 +527,13 @@ class Keyboard(AEInput.SystemInput):
       # ignore keys we know we synthesized
       self.synth_release.pop(0)
       return
-    if self.last_press:
+    elif code in self.synth_press:
+      # ignore releases for presses we're waiting to synth
+      # but make sure to synth our own release
+      self._sendKey(code, False)
+      event.consume = True
+      return
+    elif self.last_press:
       # make a copy of the current gesture before we modify it
       gesture = AEInput.Gesture(self, gesture=self.curr_gesture)
       # notify listeners of the gesture and its timestamp
@@ -658,4 +681,4 @@ class Keyboard(AEInput.SystemInput):
   AEK_NUMPAD_ENTER = 108
   AEK_THINKPAD_FN = 227
   AEK_THINKPAD_BACK = 234
-  AEK_THINKPAD_FORWARD = 233
\ No newline at end of file
+  AEK_THINKPAD_FORWARD = 233
