@@ -36,21 +36,14 @@ try:
 except ImportError:
   pass
 
-import logging, optparse, os, signal, sys, subprocess, glob, re
-import UIRegistrar, LSRConstants, AEOutput
+import optparse, os, signal, sys, subprocess, glob, re
+import UIRegistrar, LSRConstants, AEOutput, LSRLog
 from i18n import _
 from SettingsManager import SettingsManager
 from LSRConstants import HOME_USER, HOME_DIR, PROG_VERSION, PROG_DATE, \
      TEMPLATE_DIR, DEFAULT_ASSOCIATIONS, DEFAULT_PROFILE
 from LSRInterfaces import implements
 
-# define the format for log entries and timestamps on each entry
-# see http://docs.python.org/lib/node352.html
-log_format = '[%(name)s %(levelname)s %(asctime)s] %(message)s'
-time_format = '%d%m%YT%H:%M:%S'
-
-log = logging.getLogger('LSRMain')
-
 # define a regular expression for doing word wrap on the command line
 wrap_rx=re.compile(u"([\u2e80-\uffff])", re.UNICODE)
 
@@ -77,46 +70,6 @@ def wrap(text, width=79, encoding='utf-8', indent=4):
                 wrap_rx.sub(r'\1\0 ', unicode(text,encoding)).split(' ')
             ).replace('\0', '').encode(encoding)
 
-class PrintLogger(object):
-  '''
-  Provides a dirt-simple interface compatible with stdout and stderr. When
-  assigned to sys.stdout or sys.stderr, an instance of this class redirects 
-  print statements to the logging system. This means the result of the 
-  print statements can be silenced, sent to a file, etc. using the command
-  line options to LSR. The log level used is defined by the L{LEVEL} constant
-  in this class.
-  
-  @cvar LEVEL: Logging level for writes directed through this class
-  @type LEVEL: integer
-  @ivar log: Reference to the Print log channel
-  @type log: logging.Logger
-  '''
-  LEVEL = 15
-  
-  def __init__(self):
-    '''
-    Create the logger.
-    '''
-    self.log = logging.getLogger('Print')
-    self.chunks = []
-    
-  def write(self, data):
-    '''
-    Write the given data at the debug level to the logger. Stores chunks of
-    text until a new line is encountered, then sends to the logger.
-    
-    @param data: Any object that can be converted to a string
-    @type data: stringable
-    '''
-    s = data.encode('utf-8')
-    if s.endswith('\n'):
-      self.chunks.append(s[:-1])
-      s = ''.join(self.chunks)
-      self.log.log(self.LEVEL, s)
-      self.chunks = []
-    else:
-      self.chunks.append(s)
-
 def welcome():
   '''
   Prints copyright, license, and version info.
@@ -422,11 +375,7 @@ def run(options):
   import AccessEngine
   # use the profile specified on the command line or the default
   ae = AccessEngine.AccessEngine(profile, not options.no_intro)
-  # create a print log channel for redirecting print statements to debug
-  stdout = sys.stdout
-  sys.stdout = PrintLogger()
   rv = ae.run()
-  sys.stdout = stdout
   return rv
 
 def initProfiles(options):
@@ -442,7 +391,6 @@ def initProfiles(options):
   @raise Exception: When anything goes wrong during the initialization of the
     default profiles
   '''
-  log.debug('creating profiles in ' + HOME_USER)
   for profile, uies in DEFAULT_ASSOCIATIONS.items():
     for uie in uies:
       try:
@@ -567,6 +515,8 @@ def parseOptions():
                 help=LSRConstants.HELP_LOG_CHANNEL % '[Perk, Tier, ...]')
   op.add_option('--log-file', dest='log_file', action='store', type='str',
                 help=LSRConstants.HELP_LOG_FILENAME)
+  op.add_option('--log-conf', dest='log_conf', action='store', type='str',
+                help=LSRConstants.HELP_LOG_CONFIG)
   op.add_option('--init-global', action='callback', callback=getAction, 
                 default=initGlobal, help=LSRConstants.HELP_INIT_GLOBAL)
   op.add_option('--init-profiles', action='callback', callback=getAction, 
@@ -578,41 +528,6 @@ def parseOptions():
   (options, args) = op.parse_args()
   return options
 
-def configLogging(options):
-  '''
-  Configures the logging system based on the setting specified on the command
-  line. Defaults to logging exceptions only to stdout.
-  
-  @param options: Parsed options
-  @type options: optparse.Values
-  '''  
-  if options.log_level is None:
-    # default to logging critical messages only
-    options.log_level = 'info'
-  if options.log_level == 'print':
-    # set the log level to print statements if specified
-    level = PrintLogger.LEVEL
-  else:
-    # get the logging constant from the logging module by name; it's an
-    # uppercase version of the log_level string provided on the command line
-    level = getattr(logging, options.log_level.upper())
-  # get the root logger
-  root = logging.getLogger()
-  # log to a file or to stderr
-  if options.log_file:
-    handler = logging.FileHandler(options.log_file)
-  else:
-    handler = logging.StreamHandler(sys.stderr)
-  # set the format and handler for messages
-  formatter = logging.Formatter(log_format, time_format)
-  root.addHandler(handler)
-  root.setLevel(level)
-  handler.setFormatter(formatter)
-  # add filters to the root handler
-  if options.log_channels is not None:
-    for name in options.log_channels:
-      handler.addFilter(logging.Filter(name))
-
 def main():
   '''
   Print the welcome message, parses command line arguments, and configures the
@@ -627,9 +542,7 @@ def main():
   welcome()
   # parse the command line options
   options = parseOptions()
-  # set up the logging system
-  configLogging(options)
-  
+
   # check if the default profiles need to be created
   if not SettingsManager.hasDefaultProfiles():
     initProfiles(None)
@@ -645,6 +558,9 @@ def main():
     if not options.no_a11y:
       # make sure accessibility is enabled on the desktop and for LSR
       configA11y()
+    # set up the logging system
+    LSRLog.configure(options.log_conf, options.log_level, options.log_channels,
+                     options.log_file)
     # run the AccessEngine
     mtd(options)
   else:
