@@ -2,29 +2,6 @@
 Maps the commands registered in L{DefaultPerk} to keys with Alt-Shift as the
 standard modifier.
 
-Keys registered
-
-Left and Right Ctrl: Mute/unmute output
-Ctrl: Stop output
-Alt-Shift-PageUp: Increase speech rate
-Alt-Shift-PageDown: Decrease speech rate
-Alt-Shift-F1: Where Am I?
-Alt-Shift-5: Focus to pointer
-Alt-Shift-3: Pointer to focus
-Alt-Shift-Y: Read top of view (e.g. window caption)
-Alt-Shift-D: Read accessible description
-Alt-Shift-B: Read text attributes
-Alt-Shift-U: Previous item
-Alt-Shift-I: Current item
-Alt-Shift-O: Next item
-Alt-Shift-J: Previous word
-Alt-Shift-K: Current word
-Alt-Shift-L: Next word
-Alt-Shift-M: Previous character
-Alt-Shift-,: Current character
-Alt-Shift-.: Next character
-Alt-Shift-P: Show the PerkChooser for this Tier
-
 @author: Peter Parente
 @organization: IBM Corporation
 @copyright: Copyright (c) 2005, 2006 IBM Corporation
@@ -62,8 +39,8 @@ class AltShiftPerk(Perk.Perk):
     pairs = [[kbd.AEK_ALT_L, kbd.AEK_SHIFT_L], 
              [kbd.AEK_ALT_R, kbd.AEK_SHIFT_R]]
     for pair in pairs:
-      self.registerCommand(kbd, 'increase rate', False,pair+[kbd.AEK_PAGE_UP])
-      self.registerCommand(kbd, 'decrease rate',False,pair+[kbd.AEK_PAGE_DOWN])
+      self.registerCommand(kbd, 'increase rate', False,pair+[kbd.AEK_T])
+      self.registerCommand(kbd, 'decrease rate',False,pair+[kbd.AEK_G])
       self.registerCommand(kbd, 'previous item', False, pair+[kbd.AEK_U])
       self.registerCommand(kbd, 'current item', False, pair+[kbd.AEK_I])
       self.registerCommand(kbd, 'next item', False, pair+[kbd.AEK_O])
@@ -73,13 +50,13 @@ class AltShiftPerk(Perk.Perk):
       self.registerCommand(kbd, 'previous char', False, pair+[kbd.AEK_M])
       self.registerCommand(kbd, 'current char', False, pair+[kbd.AEK_COMMA])
       self.registerCommand(kbd, 'next char', False, pair+[kbd.AEK_PERIOD])
-      self.registerCommand(kbd, 'where am i', False, pair+[kbd.AEK_F1])
-      self.registerCommand(kbd, 'focus to pointer', False, pair+[kbd.AEK_5])
-      self.registerCommand(kbd, 'pointer to focus', False, pair+[kbd.AEK_3])
+      self.registerCommand(kbd, 'where am i', False, pair+[kbd.AEK_1])
+      self.registerCommand(kbd, 'focus to pointer', False, pair+[kbd.AEK_R])
+      self.registerCommand(kbd, 'pointer to focus', False, pair+[kbd.AEK_F])
       self.registerCommand(kbd, 'read top', False, pair+[kbd.AEK_Y])
       self.registerCommand(kbd, 'read text attributes',False, pair+[kbd.AEK_B])
       self.registerCommand(kbd, 'read description', False, pair+[kbd.AEK_D])
-      self.registerCommand(kbd, 'read all', False, pair+[kbd.AEK_A])
-      self.registerCommand(kbd, 'show perk chooser', False, pair+[kbd.AEK_P])
+      self.registerCommand(kbd, 'read all', False, pair+[kbd.AEK_H])
+      self.registerCommand(kbd, 'show perk chooser', False, pair+[kbd.AEK_A])
       self.registerCommand(kbd, 'show settings chooser', False, 
-                           pair+[kbd.AEK_S])
+                           pair+[kbd.AEK_Q])
